import urllib.parse
import urllib.request
import xml.etree.ElementTree as ET
import hashlib
from resources.lib.md5crypt import unix_md5_crypt
import xbmc
import os
import xbmcvfs
import uuid
import xbmc

API_URL = "https://webshare.cz/api"

DEVICE_ID_FILE = os.path.join(xbmcvfs.translatePath("special://profile/addon_data/plugin.video.webshare/"), "device_id.txt")
def log(message):
    xbmc.log(f"[Webshare] {message}", level=xbmc.LOGINFO)
def get_device_uuid():
    if xbmcvfs.exists(DEVICE_ID_FILE):
        with xbmcvfs.File(DEVICE_ID_FILE) as f:
            return f.read()
    new_id = str(uuid.uuid4())
    with xbmcvfs.File(DEVICE_ID_FILE, 'w') as f:
        f.write(new_id)
    return new_id
def search(what, sort=None, limit=None, offset=None, category=None):
    data = {
        'what': what,
    }
    if sort:
        data['sort'] = sort
    if limit is not None:
        data['limit'] = str(limit)
    if offset is not None:
        data['offset'] = str(offset)
    if category:
        data['category'] = category

    encoded = urllib.parse.urlencode(data).encode('utf-8')
    headers = {
        'Accept': 'text/xml; charset=UTF-8',
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
    }

    req = urllib.request.Request(API_URL+"/search/", data=encoded, headers=headers)
    with urllib.request.urlopen(req) as response:
        xml_data = response.read()

    return parse_response(xml_data)

def parse_response(xml_data):
    root = ET.fromstring(xml_data)
    results = []
    for file in root.findall('file'):
        results.append({
            'id': file.findtext('ident'),
            'name': file.findtext('name'),
            'type': file.findtext('type'),
            'img': file.findtext('img'),
            'size': file.findtext('size'),
            'votes_pos': file.findtext('positive_votes'),
            'votes_neg': file.findtext('negative_votes'),
            'password': file.findtext('password'),
        })
    return results

def load_item_link(ident, password=None, download_type="video_stream", token=None):
    import platform

    width = xbmc.getInfoLabel("System.ScreenWidth")
    height = xbmc.getInfoLabel("System.ScreenHeight")
    device_res_x = int(width) if width.isdigit() else 1920
    device_res_y = int(height) if height.isdigit() else 1080

    vendor = platform.system()
    model = platform.node()
    id = get_device_uuid()
    log(f"UUID: {id}")
    log(f"x: {device_res_x} y: {device_res_y} vendor: {vendor} model: {model}")
    url = "https://webshare.cz/api/file_link/"
    data = {
        'ident': ident,
        'download_type': download_type,
        'device_uuid': id,
        'device_vendor': vendor,
        'device_model': model,
        'device_res_x': str(device_res_x),
        'device_res_y': str(device_res_y),
        'force_https': "1",
        'wst': str(token),
    }

    encoded = urllib.parse.urlencode(data).encode('utf-8')
    headers = {
        'Accept': 'text/xml; charset=UTF-8',
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
    }

    req = urllib.request.Request(url, data=encoded, headers=headers)
    with urllib.request.urlopen(req) as response:
        xml_data = response.read()

    root = ET.fromstring(xml_data)
    status = root.findtext('status')
    if status == 'OK':
        return root.findtext('link')
    else:
        error_code = root.findtext('code')
        error_msg = root.findtext('message')
        raise Exception(f"Webshare error: {error_code} – {error_msg}")



def get_user_salt(username_or_email):
    url = f"{API_URL}/salt/"
    data = urllib.parse.urlencode({'username_or_email': username_or_email}).encode('utf-8')
    headers = {
        'Accept': 'text/xml; charset=UTF-8',
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
    }

    req = urllib.request.Request(url, data=data, headers=headers)
    with urllib.request.urlopen(req) as response:
        xml_data = response.read()

    root = ET.fromstring(xml_data)
    status = root.findtext('status')
    if status == 'OK':
        xbmc.log(f"Salt {root.findtext('salt')}", level=xbmc.LOGINFO)
        return root.findtext('salt')
    elif status == 'FATAL':
        code = root.findtext('code')
        message = root.findtext('message')
        raise Exception(f"Salt fetch failed: {code} - {message}")
    else:
        raise Exception("Unexpected response format from salt endpoint")

def compute_password_digest(password, salt):
    md5_crypted = unix_md5_crypt(password, salt)

    sha1 = hashlib.sha1(md5_crypted.encode('utf-8')).hexdigest()
    return sha1

def login(username_or_email, password, keep_logged_in=0):
    try:
        salt = get_user_salt(username_or_email)
        log(f"password: {password}")
        log(f"username_or_email: {username_or_email}")
        password_digest = compute_password_digest(password, salt)

        url = f"{API_URL}/login/"
        post_data = {
            'username_or_email': username_or_email,
            'password': password_digest,
            'keep_logged_in': str(keep_logged_in)
        }
        data = urllib.parse.urlencode(post_data).encode('utf-8')
        headers = {
            'Accept': 'text/xml; charset=UTF-8',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
        }

        req = urllib.request.Request(url, data=data, headers=headers)
        with urllib.request.urlopen(req) as response:
            xml_data = response.read()

        root = ET.fromstring(xml_data)
        status = root.findtext('status')
        if status == 'OK':
            token = root.findtext('token')
            return token
        elif status == 'FATAL':
            code = root.findtext('code')
            message = root.findtext('message')
            raise Exception(f"Login failed: {code} - {message}")
        else:
            raise Exception("Unexpected response format from login endpoint")

    except Exception as e:
        raise Exception(f"Login error: {str(e)}")
